/* Copyright IBM Corp. 2008  All Rights Reserved.                    */
dojo.provide( "com.ibm.mashups.enabler.model" );

// general
dojo.provide("com.ibm.mashups.enabler.Identifiable");
dojo.provide("com.ibm.mashups.enabler.Localized");
dojo.provide("com.ibm.mashups.enabler.MetaData");
dojo.provide("com.ibm.mashups.enabler.Locator");
dojo.provide("com.ibm.mashups.enabler.Iterator");
dojo.provide("com.ibm.mashups.enabler.Logout");

// strategies
dojo.provide("com.ibm.mashups.enabler.strategy.Strategy");
dojo.provide("com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy");
dojo.provide("com.ibm.mashups.enabler.strategy.TreeLoadAheadStrategy");

// models
dojo.provide("com.ibm.mashups.enabler.model.Model");
dojo.provide("com.ibm.mashups.enabler.model.ListModel");
dojo.provide("com.ibm.mashups.enabler.model.TreeModel");
dojo.provide("com.ibm.mashups.enabler.model.NavigationModel");
dojo.provide("com.ibm.mashups.enabler.model.LayoutModel");
dojo.provide("com.ibm.mashups.enabler.model.CatalogCategoryModel");
dojo.provide("com.ibm.mashups.enabler.model.CatalogElementModel");
dojo.provide("com.ibm.mashups.enabler.model.CommunityModel");

// nodes
dojo.provide("com.ibm.mashups.enabler.navigation.NavigationNode");
dojo.provide("com.ibm.mashups.enabler.navigation.SharedNavigationNode");
dojo.provide("com.ibm.mashups.enabler.navigation.SharedNavigationRoot");
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogNode");
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogCategory");
dojo.provide("com.ibm.mashups.enabler.catalog.CatalogElement");
dojo.provide("com.ibm.mashups.enabler.layout.LayoutNode");
dojo.provide("com.ibm.mashups.enabler.layout.LayoutFragment");
dojo.provide("com.ibm.mashups.enabler.community.Community");

/**
 * Base interface for all classes which handle actions asynchronously.
 * An action may involve one or more operations, for example creating or
 * updating a model resource, which in turn may involve one or more HTTP
 * requests. The Deferred interface allows to trigger a callback after
 * the entire action, i.e. all involved HTTP requests, is finished. Thus,
 * the Deferred interface may be used to get the overall result of an action.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.Deferred", null, {
    /**
     * @private
     */
    constructor:function () {
    },

    /**
     * Sets the handler of the deferred action. It is called when the entire
     * action has finished.
     * @param {Function(Object resource, int statusCode, Object[] params)} callback
     * the callback funtion. Must not be <code>null</code><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<b>Callbackparameters</b><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>resource</code> - resource object or i string
     * of the related resource.
     * May be <code>null</code> in case the action does not relate
     * to a particular resource. In case multiple resources are involved in the
     * action, the DeferredOperation interface may be used to obtain the resource
     * objects or string <code>id</code>s of those resources.<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>statusCode</code> - the overall HTTP status
     * code of the action (the hightest status code of the involved operations).<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>params</code> - the parameters
     * passed into the callback
     * @param {Object[]} parameters optional array of parameters to be
     * passed on to the callback function. May be <code>null</code>
     * @type void
     */
    setFinishedCallback: function(callback, parameters) {
    },

    /**
     * Executes the deferred action
     * @param {Boolean} sync optional parameter. If true, the deferred
     * action is executed synchronously, otherwise asynchronously.
     * In the latter case the callback functions are called as soon as
     * the operation is finished. Defaults to true.
     * @return {Object} if called asynchnrosously the return value is <code>null</code>,
     * otherwise a JSON object with the following format<br>
     * <code>&#123; returnValue&nbsp;: </code>value of the actual method
     * being called if the operation executed successfully, otherwise
     * <code>null</code>,<br>
     * <code>&nbsp;&nbsp;errorCode&nbsp;&nbsp;&nbsp;: </code>the HTTP
     * error code if the operation executed with a failure, otherwise
     * <code>null.&#125; </code>
     */
    start: function(sync) {
        return {};
    }
});

/**
 * In addition to the Deferred interface, which allows to trigger callbacks after the
 * entire action  has finished, the DeferredOperation interface allows to trigger a
 * callback for each involved operation.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.DeferredOperation", com.ibm.mashups.enabler.Deferred, {

    OPERATION_GET: "GET",
    OPERATION_CREATE: "CREATE",
    OPERATION_MODIFY: "MODIFY",
    OPERATION_DELETE: "DELETE",

    /**
     * @private
     */
    constructor:function () {
    },

    /**
     * Sets the handler of the deferred action. The handler is called for each
     * operation the action involves.
     * @param {Function(Object node, string mode, int statusCode, Object[] params)} callback
     * the callback funtion. Must not be <code>null</code><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<b>Callbackparameters</b><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>node</code> - resource object or string id of the
     * related resource, depending on the operation. For example, when a resource is created, the
     * resource itself is returned. In case of a delete operation, the id is returned
     * instead.
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>mode</code> - the mode of the operation.
     * May be one of <code>OPERATION_GET</code>, <code>OPERATION_CREATE</code>,
     * <code>OPERATION_MODIFY</code>, or <code>OPERATION_DELETE</code><br>.
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>statusCode</code> - the overall HTTP status code
     * of the operation.<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>params</code> - the parameters
     * passed into the callback
     * @param {Object[]} parameters optional array of parameters to be
     * passed on to the callback function. May be <code>null</code>
     * @type void
     */
    setOperationCallback: function(callback, parameters) {
    }
});

/**
 * The DeferredIterator can be used to iterate over a list of objects 
 * in an asynchronous fashion. The callback handler is called as 
 * soon as the next object has been loaded.
 * The start method is only used for the asynchronous aspect of 
 * the deferred iterator. For the synchronous aspect the methods
 * such as hasNext() can be called directly.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.DeferredIterator", [com.ibm.mashups.enabler.Deferred, com.ibm.mashups.enabler.Iterator], {
  /**
      * @private
      */
    constructor:function () {
    },
    /**
     * Sets the handler of the deferred action. It is called when the
     * next object in the list has been loaded and is ready to be processed.
     * @param {Function(object nextElement, Object[] params)} callback
     * the callback funtion. Must not be <code>null</code><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<b>Callbackparameters</b><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>nextElement</code> - the next 
     * object in the list<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>params</code> - the parameters 
     * passed into the addForEachCallback
     * @param {Object[]} parameters optional array of parameters to be 
     * passed on to the callback function
     * @returns {Boolean} true if the loop should continue and call this method 
     * again for the next object (if there is one) or<br>false when the 
     * loop should be interrupted
     */
    setForEachCallback: function(callback, parameters) {
        return new Boolean();
    }
});

/**
 * Interface for any resource in the system that can be identified
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.Identifiable", null, {
  /**
      * @private
      */
    constructor:function () {
    },
    /**
     * Returns the ID of the resource that implements the <code>Identifiable</code> interface.
     * @return {String} the identifier; never <code>null</code>
     */
    getID: function() {
        return String(); 
    }
});

/**
 * Interface providing methods to obtain title and description of a
 * resource. Please note that title or description are only returned if they
 * are available in the requested locale. No fallback for locales is performed.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.Localized", null, {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * Returns a list model containing the locales that are supported by this
     * object. The presence of a locale in this list does not mean that a title
     * <b>and</b> description is available, but rather that either one or both are available in that locale.
     * @return {Array} a list of locales defined for this object, returns an 
     * empty array if no locales are supported. Can never be <code>null</code>.
     */
    getLocales: function() {
        return [];
    },

    /**
     * Returns the title of this object in the given locale.
     * @param {String} locale the locale for which to retrieve the title, 
     * must not be <code>null</code>.
     * @return {String} the title of this node in the given locale. If a title is not
     * available in the given locale, this method will return <code>null</code>. 
     * It is up to the invoker of the method to implement an appropriate fallback mechanism.
     */
    getTitle: function(locale) {
        return new String(); 
    },

    /**
     * Sets the title for the given locale.
     * @param {String} title    title to set; must not be <code>null</code>
     * @param {String} locale   locale to set the title for; must not be <code>null</code>
     * @return {String} the former title; if none existed, <code>null</code> is returned
     */
    setTitle: function(title, locale) {
        return new String(); 
    },

    /**
     * Returns the description of this object in the given locale.
     * @param {String} locale   the locale for which to retrieve the 
     * description, must not be <code>null</code>.
     * @return {String} the description of this node in the given locale. If a 
     * description is not available in the given locale, this method 
     * will return <code>null</code>. It is up to the invoker of the 
     * method to implement an appropriate fallback mechanism.
     */
    getDescription: function(locale) {
        return new String(); 
    },

    /**
     * Sets the description for the given locale.
     * @param {String} desc     description to set; must not be <code>null</code>
     * @param {String} locale   locale to set the description for; must not be <code>null</code>
     * @return {String} the former description; if none existed, <code>null</code> is returned
     */
    setDescription: function(desc, locale) {
        return new String(); 
    }
});

/**
 * An interface describing meta data associated with a resource.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.MetaData", null, {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * Returns a list of the names of meta data entries.
     * @return {String[]} an array containing the names; never <code>null</code> 
     */
    getMetaDataNames: function() {
        return []; 
    },

    /**
     * Returns the value of a meta data entry for the given parameter name.
     * @param {String} name the meta data name; must not be <code>null</null>
     * @return {String} the value of the meta data entry for the given name, or <code>null</code> if none exists
     */
    getMetaData: function(name) {
        return new String();
    },

    /**
     * Sets the value for the meta data identified by the specified name.
     * @param {String} name     name of meta data to set the value for; must not be <code>null</null>
     * @param {String} value    meta data value to set; must not be <code>null</null>
     * @return {String} the former value for the name; if none existed, <code>null</code> is returned
     */
    setMetaData: function(name, value) {
        return new String(); 
    },

    /**
     * Removes the meta data identified with the specified name.
     * @param{String} name  name of meta data to remove; must not be <code>null</null>
     * @return {String} the former value for the name; if none existed, <code>null</code> is returned
     */
    removeMetaData: function(name) {
        return new String();
    }
});

/**
 * This interface defines search methods that allow locating resources in a model.
 * A resource is identified by its ID (mandatory). This interface is the base for all locators;
 * other locators allow a more specific search in the context they are offered in, 
 * e.g. a search for a node with certain properties in a model.<br>
 * A <code>Locator</code> is usually obtained using the <code>getLocator()</code> method .
 * <pre>
 * var treeModel model = ...;
 * var locator = model.getLocator();
 * var node = locator.find(...);
 * </pre>
 * This locator returns an {@code Object} which is part of some model. 
 * What exact kind of object is returned depends on the locator.
 * Please refer to individual JavaScriptDoc for the locator used. 
 * The result object can be expected to be an element of the model
 * the locator is used on unless specified otherwise.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.Locator", null,     {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * Returns an element of a model with the given ID.
     * @param {String} id   the id or uri of the object to find 
     * (without any scope). Must not be <code>null</code>.
     * @return {Deferred(Object)} a deferred object used to start this operation. 
     * The return value when executed through the deferred object is
     * the element with the given ID or <code>null</code> if the element cannot be found.
     */
    find: function (id) {
        return new Deferred();
    }
});

/**
 * Base class for all models
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.model.Model", null, {
  /**
      * @private
      */
    constructor:function () {
    },
    
    /**
     * Sets the specified array of strategies or in case none has been 
     * defined falls back to the default strategy
     * @param {Strategy[]}  strategy    array of strategies to set;
     *                                  may be <code>null</code>
     * @type void
     */
    setStrategy: function (strategy) {
    }
});

/**
 * Extended iterator interface that allows iterating over elements 
 * in a sequentiell order with the possibility to set the starting 
 * point of the iteration as well as the size of the list.
 * 
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.Iterator", null, {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * Returns <tt>true</tt> if the iteration has more elements.
     * @return  {Boolean} <tt>true</tt> if the iterator has more elements, 
     *          otherwise <tt>false</tt>.
     */
    hasNext: function () {
        return new Boolean(); 
    },

    /**
     * Returns the next element in the iteration.  Calling this method
     * repeatedly until the hasNext() method returns false will return 
     * each element in the underlying collection exactly once.
     * The cursor position is increased by one . if no element is available 
     * at the specified position, <code>null</code> is returned and the 
     * position is not changed
     * 
     * @returns  {Object} the next element in the iteration
     */
    next: function () {
        return {};
    },

    /**
     * Returns the number of elements in this iterator.
     * @return {int} the number of elements in this iterator
     */
    size: function () {
        return null;
    },

    /**
     * Sets the zero-based position of the cursor, i.e the last 
     * element is addressed through size()-1.
     * A position which is out of the bounds of the current iteration 
     * is ignored and in this case the position is set to the nearest 
     * possible value within valid bounds.
     * 
     * @param {int} position    position of the cursor. Defaults to zero
     * @type void
     */
    setCursorPosition: function (position) {
    },

    /**
     * Returns the zero-based position of the cursor, i.e. zero for the first element
     * 
     * @return {int} the position of the cursor
     */
    getCursorPosition: function () {
        return null;
    }
});

/**
 * A model representing a list. acting as a base class for all concrete list models
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.ListModel", [com.ibm.mashups.enabler.model.Model, com.ibm.mashups.enabler.Locator], {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * Returns an interator over elements of the list.
     * 
     * @type    DeferredIterator
     * @return  a deferred iterator
     */
    iterator: function () {
        return new Iterator(); 
    },

    /**
     * Inserts the specified node into the list model at the specified 
     * position; the node must be created with the create method of the 
     * concrete ListModel
     * 
     * @param {Object} node     node or node uri (without any scope) to
     *                          insert into the list model. Must not be 
     *                          <code>null</code>
     * @param {Object} nextNode node object or node uri (without any scope) of 
     *                          the successor node before which the node 
     *                          is to be inserted;
     *                          if <code>null</code> is specified, the 
     *                          node is appended at the end of the existing nodes
     * @return {void}
     */
    insert: function (node, nextNode) {
    },

    /**
     * removes the specified node from the list model
     * 
     * @param{Object} node  node object or node uri (without any scope). 
     *                      Must not be <code>null</code>
    * @type void
     */
    remove: function (node) {
    }
});

/**
 * This interface provides a generic tree model acting as a base class 
 * for all concrete tree models
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.TreeModel", [com.ibm.mashups.enabler.model.Model, com.ibm.mashups.enabler.Locator], {
  /**
      * @private
      */
     constructor:function () {
    },

    /**
     * Returns the root node of the model; may return <code>null</code> 
     * if the concrete tree model supports for empty models
     * 
     * @type    Deferred(Object)
     * @return  a deferred object used to start this operation. The 
     *          return value when executed through the deferred object 
     *          is the root object of the tree model
     */
    getRoot: function () {
        return new Deferred();
    },

    /**
     * Returns whether or not the given node has children. Please pay 
     * attention to the documentation of the return value; it is not 
     * guaranteed that children are present if the method returns 
     * <code>true</code> - the iterator might be empty.
     * 
     * @param   {Object} node   node object or node uri (without any scope)
     *                          for which to check if it has children. Must 
     *                          not be <code>null</code>.
     * @type    Boolean
     * @return  <tt>true</tt> if the specified node has children, <tt>false</tt> otherwise  
     */
    hasChildren: function (node) {
        return new Boolean();
    },

    /**
     * Returns an iterator over the child elements for the given node.
     * 
     * @param {Object} node node object or node uri (without any scope) 
     *                      for which to return its children. Must not 
     *                      be <code>null</code>.
     * @type    com.ibm.mashups.enabler.DeferredIterator
     * @return  deferred iterator for the children of the given node. 
     *          Never <code>null</code>
     */
    getChildren: function (node) {
        return new com.ibm.mashups.enabler.DeferredIterator(); 
    },

    /**
     * Returns the parent of a given node.
     * @param {Object} node node object or node uri (without any scope) 
     *                      for which to return its parent. Must not 
     *                      be <code>null</code>.
     * @type    Deferred(Object)
     * @return  a deferred object used to start this operation. The 
     *          return value when executed through the deferred object 
     *          is the parent of the given node, or <code>null</code> 
     *          if the node has no parent
     */
    getParent: function (node) {
        return new Deferred();
    },

    /**
     * Inserts a node into the tree model at the specified location; 
     * the node must either be created with the create method of the 
     * concrete tree model before, or must exist in the tree model; 
     * in this case the existing node is moved to the specified location
     * 
     * @param {Object} node         node to insert or move. Must not be 
     *                              <code>null</code>.
     * @param {Object} parentNode   node object or node uri (without any 
     *                              scope) of the parent node. The given 
     *                              node is inserted/moved underneath the 
     *                              parent node; must not be <code>null</code>.
     * @param {Object} nextNode     node object or node uri (without any 
     *                              scope) of the successor node before 
     *                              which the node is to be inserted; 
     *                              if <code>null</code> is specified, 
     *                              the node is appended at the end of 
     *                              the existing nodes
     * @type void
     */
    insert: function (node, parentNode, nextNode) {
    },

    /**
     * Deletes a node from the model. All its children are deleted as well.
     * 
     * @param {Object} node node object or node uri (without any scope) 
     *                      to delete from the model. Must not be <code>null</code>.
     * @type void
     */
    remove: function (node) {
    }
});

/**
 * Interface for a navigation model. This tree model describes the topology in 
 * which the navigation is structured.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.model.NavigationModel", com.ibm.mashups.enabler.TreeModel, {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * Returns the layout model for the given page.
     * @param {NavigationNode} node the NavigationNode or, if node is a string, 
     * the uri of the node for which to return its model. The page must be part 
     * of this model. Must not be <code>null</code>.
     * @type com.ibm.mashups.enabler.LayoutModel
     * @return a layout model or <code>null</code> if none exists. 
     */
    getLayoutModel: function (node) {
        return new com.ibm.mashups.enabler.LayoutModel();
    },

    /**
     * Returns a special node of the model defining the root of all pages that 
     * are shared with a user. May be <code>null</code> if the navigation tree is empty
     * @type Deferred(Object)
     * @return a deferred object used to start this operation. The return value 
     * when executed through the deferred object is the shared root object of the tree model
     */
    getSharedRoot: function () {
        return new Deferred();
    },

    /**
     * creates a new navigation node object. The created node can be inserted 
     * into the model using an appropriate <code>insert</code> method defined 
     * on a subinterface of this interface. The node will not appear in the model 
     * unless it is inserted.
     * @param {object{name: value, ...}} context array of predefined information 
     * used for the creation of the node. May be <code>null</code>. Accepted 
     * names are:<br>
     * - xyz - value: TODO
     * @type com.ibm.mashups.enabler.navigation.NavigationNode
     * @return the created node
     */
    create: function (context) {
        return new com.ibm.mashups.enabler.navigation.NavigationNode();
    },

    /**
     * Commits the modifications applied to this model and all dependant models<br>
     * After the model is committed successfully, this method must not be called 
     * again. The same is true for any other method modifying the model or its nodes.
     * @type Deferred
     * @return a deferred object used to start this operation. The return value 
     * when executed through the deferred object is <code>null</null>
     */
    commit: function () {
        return new Deferred();
    }
});

/**
 * This interface provides a tree model for the layout of a page. As of now the 
 * only node within a layout model is a fragment node.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.model.LayoutModel", com.ibm.mashups.enabler.TreeModel, {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * creates a new layout node object. The created node can be inserted into 
     * the model using an appropriate <code>insert</code> method. The node will 
     * not appear in the model unless it is inserted.
     * @param {object{name: value, ...}} context array of predefined information 
     * used for the creation of the node. May be <code>null</code>. Accepted names are:<br>
     * - xyz - value: TODO
     * @type com.ibm.mashups.enabler.layout.LayoutNode
     * @return the created node
     */
    create: function (context) {
        return new com.ibm.mashups.enabler.layout.LayoutNode();
    }
});

/**
 * Interface representing a navigation node
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.navigation.NavigationNode", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.Localized, com.ibm.mashups.enabler.MetaData],     {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * Returns the community ID, or <code>null</code> if no community has been 
     * associated with the node.
     * @type LayoutNode 
     * @return the community ID, or <code>null</code> if not has been associated
     */
    getCommunity: function() {
        return new String();
    }
});

/**
 * Marker interface representing a shared navigation node
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.navigation.SharedNavigationNode", com.ibm.mashups.enabler.navigation.NavigationNode,     {
  /**
      * @private
      */
    constructor:function () {
    }
});

/**
 * Marker interface representing a shared navigation node
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.navigation.SharedNavigationRoot", com.ibm.mashups.enabler.navigation.NavigationNode,     {
  /**
      * @private
      */
    constructor:function () {
    }
});

/**
 * An interface for the nodes of a LayoutModel.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.layout.LayoutNode", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.MetaData],     {
  /**
      * @private
      */
    constructor:function () {
    }
});

/**
 * Interface that represents a layout fragment
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.layout.LayoutFragment", [com.ibm.mashups.enabler.Localized, com.ibm.mashups.enabler.layout.LayoutNode],     {
  /**
      * @private
      */
    constructor:function ( ) {
    },

    /**
     * Returns the content of the fragment as a string.
     * @type String
     * @return the fragment content. Never <code>null</code>
     */
    getFragment: function() {
        return new String();
    },

    /**
     * Sets the content of the fragment. Any previously set content is replaced.
     * @param {String} content content to set. Must not be <code>null</code>.
     * @type void
     */
    setFragment: function(content) {
    }
});

/**
 * Interface for a catalog model containing category nodes. This tree model 
 * describes the topology in which the categories are structured.
 * As of now only one level below the root is supported; no child categories.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.model.CatalogCategoryModel", com.ibm.mashups.enabler.TreeModel,     {
  /**
      * @private
      */
    constructor:function ( ) {
    },

    /**
     * Returns the catalog entry model for the given category.
     * @param {CatalogCategoryNode} category the CatalogCategoryNode for which 
     * to return the CatalogEntryModel; must not be <code>null</code>
     * @type CatalogEntryModel 
     * @return catalog entry model for given category, Never <code>null</code>. 
     */
    getCatalogEntryModel: function (category) {
        return new CatalogEntryModel(); 
    },

    /**
     * creates a new CatalogCategoryNode. The created node can be inserted into 
     * the model using an appropriate <code>insert</code> method. The node will 
     * not appear in the model unless it is inserted.
     * @param {object{name: value, ...}} context array of predefined information 
     * used for the creation of the node. May be <code>null</code>. Accepted names are:<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>category</code> - the name of the category 
     * to be created (mandatory)<br>
     * @type CatalogCategoryNode 
     * @return the created node 
     */
    create: function (context) {
        return new CatalogCategoryNode();
    },

    /**
     * Commits the modifications applied to this model and all dependant models.<br>
     * After the model is committed successfully, this method must not be called 
     * again. The same is true for any other method modifying the model or its nodes.
     * @type Deferred 
     * @return a deferred object used to start this operation. The return value 
     * when executed through the deferred object is <code>null</null>
     */
    commit: function () {
        return new Deferred();
    }
});

/**
 * Interface for a catalog model containing entry nodes. This list model 
 * describes a list of concrete catalog entries for a given category.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.model.CatalogEntryModel", com.ibm.mashups.enabler.ListModel, {
  /**
      * @private
      */
    constructor:function () {
    },

    /**
     * creates a new CatalogEnryNode. The created node can be inserted into the
     * model using an appropriate <code>insert</code> method. The node will not 
     * appear in the model unless it is inserted.
     * @param {object{name: value, ...}} context array of predefined information 
     * used for the creation of the node. May be <code>null</code>. Accepted 
     * names are:<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>cid</code> - the client id for the entry 
     * to be created (optional)<br>
     * @type CatalogEntryNode 
     * @return the created node
     */
    create: function (context) {
        return new CatalogEntryNode();
    }

});

/**
 * Interface that represents a catalog category; used in the CatalogCategoryModel
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.catalog.CatalogCategoryNode", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.Localized],     {
  /**
      * @private
      */
    constructor:function ( ) {
    }
});

/**
 * Interface that represents a catalog entry; used in the CatalogEntryModel
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.catalog.CatalogEntryNode", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.Localized],     {
  /**
      * @private
      */
    constructor:function ( ) {
    },

    /**
     * Returns the definition URL of this catalog entry
     * @type String 
     * @return the definition url, never <code>null</code>
     */
    getDefinitionURL: function() {
        return new String();
    },

    /**
     * Returns the icon URL of this catalog entry
     * @type String 
     * @return the icon url, never <code>null</code>
     */
    getIconURL: function() {
        return new String();
    },

    /**
     * Sets the definition URL of this catalog entry
     * @param {String} url definition url to set; must not be <code>null</code>
     * @type void
     */
    setDefinitionURL: function(url) {
    },

    /**
     * Sets the icon URL of this catalog entry
     * @param {String} url icon url to set; must not be <code>null</code>
     * @type void
     */
    setIconURL: function(url) {
    }
});

//dojo.declare( "com.ibm.mashups.enabler.model.PreferenceModel", com.ibm.mashups.enabler.TreeModel,     {
//    constructor:function ( ) {
//// summary: PreferenceModel; exposes a tree model containg hierarchical arranged preference sets
//// The tree is restricted to three levels. The first one is ADMIN, the second one INSTANCE, the third one USER.<br>
//// The only possible nodes in the tree are PreferenceSet objects
////
//},
//
//        commit: function () {
//            // summary: persists the updates performed with this and all dependant models
//             return new Deferred(); /*Deferred @return deferred object used to execute the commit function asynchronously */
//        },
//        getPreferenceSetModel: function (/*PreferenceSet*/ node ) {
//            // summary: Returns the preference set model for the given preference set. The preference set model is a layer representation of this single level
//             // node: the preference set
//            return new PreferenceSetModel(); /* PreferenceSetModel @return a preference set model*/
//        },
//        getHierarchyPreferenceSetModel: function (/*PreferenceSet*/ node ) {
//            // summary: Returns the hierarchy preference set model for the given preference set. The hierarchy preference set represents a union of all preference sets in the tree starting from the given node up to the root
//             // node: the preference set
//            return new PreferenceSetModel(); /* PreferenceSetModel @return a hierarchy preference set model*/
//        }
//    }
//);
//
//// PreferenceSet nodes
////
//dojo.declare( "com.ibm.mashups.enabler.preference.PreferenceSet", com.ibm.mashups.enabler.Identifiable,
//                 {
//        constructor:function ( ) {
////
//// summary: PreferenceSet; Interface for a set of preferences. This node is only used in the preference tree model.
////
//},
//
//        getLevel: function() {
//            // summary: Returns the level name - either ADMIN, INSTANCE or USER
//            return new String(); /* Level @return the level name - either ADMIN, INSTANCE or USER */
//        }
//    }
//);
//
//
//dojo.declare( "com.ibm.mashups.enabler.model.PreferenceSetModel", com.ibm.mashups.enabler.ListModel,     {
//    constructor:function ( ) {
////
//// summary: PreferenceSetModel; This list model describes a composite of the nodes PreferenceNode and PreferenceSetNode
//// Unless stated otherwise, return values of the list model methods are of type GenericPreferenceNode
////
//},
//
//        getPreference: function (/*String*/name) {
//            // summary: Returns the preference node with the given name. It can either be of type PreferenceNode or PreferenceSetNode.
//            // This method only locates preferences within this preference set and does not look in nested preference sets.
//            // name: the name of the preference to find. Must not be null.
//            return new GenericPreferenceNode(); /*GenericPreferenceNode @return the node matching the given name or null if not found */
//        },
//        create: function (/*object{name: value, ...}*/context) {
//            // summary: creates a new PreferenceNode or PreferenceSetNode;
//            // context: array of predefined information used for
//            // the creation of the node (also contains level ADMIN, INSTANCE or USER)
//            // &nbsp;&nbsp;&nbsp;&nbsp;<b>Contextparameters</b><br>
//            // &nbsp;&nbsp;&nbsp;&nbsp;<code>type</code> - Must be PreferenceSetNode or PreferenceNode<br>
//            // &nbsp;&nbsp;&nbsp;&nbsp;<code>name</code> - Defines the name of the new preference. Must be set.
//            //
//            return new PreferenceNode(); /* PreferenceNode/PreferenceSetNode @return created node */
//        }
//    }
//);
//
////
//// PreferenceLocator
////
//dojo.declare( "com.ibm.mashups.enabler.preference.PreferenceLocator", com.ibm.mashups.enabler.Locator,
//    {
//        findByPreferenceName: function (/*String*/name) {
//            // summary: returns the node with the specified name or null if not found
//            // name: name to lookup
//            return new Deferred(); /*Deferred/GenericPreferenceNode @return the deferred node matching the given name or null if not found */
//        }
//    }
//);
//
////
//// Generic Preference node
////
//dojo.declare( "com.ibm.mashups.enabler.preference.GenericPreferenceNode", null,
//    {
//        getName: function() {
//            // summary: Returns the name of the preference
//            return new String(); /* String @return string of the preference name */
//        },
//        isReadOnly: function() {
//            // summary: Returns whether the specified preference is readonly
//            return new Boolean(); /* bool @return true if the preference is readonly */
//        },
//        setReadOnly: function(/*bool*/ readOnly) {
//            // summary: Sets the preference, returns the former preference
//            // readOnly: sets whether the value is read only or not
//             return new Boolean(); /* boolean @return former readOnly flag of the modified preference */
//        }
//    }
//);
//
//
////
//// PreferenceSet nodes
////
//dojo.declare( "com.ibm.mashups.enabler.preference.PreferenceSetNode",
//              [com.ibm.mashups.enabler.preference.GenericPreferenceNode,com.ibm.mashups.enabler.ListModel,com.ibm.mashups.enabler.preference.PreferenceLocator],
//                 {
//        constructor:function ( ) {
////
//// summary: PreferenceSetNode; exposes a composite list model containg all preferences nodes. Each node can either be
//// of type <code>PreferenceNode</code> or <code>PreferenceSetNode</code>.
////
//},
//
//        getPreference: function (/*String*/name) {
//            // summary: Returns the preference node with the given name. It can either be of type PreferenceNode or PreferenceSetNode.
//            // This method only locates preferences within this preference set and does not look in nested preference sets.
//            // name: the name of the preference to find. Must not be null.
//            return new GenericPreferenceNode(); /*GenericPreferenceNode @return the node matching the given name or null if not found */
//        }
//    }
//);
//
////
//// Preference node
////
//dojo.declare( "com.ibm.mashups.enabler.preference.PreferenceNode", com.ibm.mashups.enabler.preference.GenericPreferenceNode,
//    {
//        getValue: function() {
//            // summary: Returns the first value of the preference
//            return new String(); /* String @return first value of the preference */
//        },
//        getValues: function() {
//            // summary: Returns the value of the preference
//            return new String(); /* String[] @return array of values of the preference */
//        },
//        setValue: function(/*String*/value) {
//            // summary: Sets the preference, returns the former value
//            // value: value to set
//            return new String(); /* string @return former value of the modified preference */
//        },
//        setValues: function(/*String[]*/values) {
//            // summary: Sets the preference, returns the former value
//            // values: value to set
//            return new String(); /* string @return former values of the modified preference */
//        }
//    }
//);

/**
 * An interface that allows the retrieval and setting of a Theme object that 
 * is associated with a resource.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.ThemeProvider", null, {
  /**
      * @private
      */
    constructor:function ( ) {
    },

    /**
     * Returns the theme associated with this element.
     * @type Deferred(Theme) 
     * @return the theme set for this element or <code>null</code> if none was set. 
     */
    getTheme: function () {
        return new Theme();
    },

    /**
     * Sets the theme.
     * @param {Theme} theme theme to set, must not be <code>null</code>.
     * @type void
     */
    setTheme: function (theme) {
    }
});

/**
 * Interface for a theme model. This list model describes a list of theme objects.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.model.ThemeModel", com.ibm.mashups.enabler.ListModel, {
  /**
      * @private
      */
    constructor:function () {
    },

   /**
   * Returns the skin model for the given theme.
   * @param {Theme} theme the Theme for which to return the skin model; 
   * must not be <code>null</code>
   * @type SkinModel 
   * @return skin model for given theme, Never <code>null</code>. 
   */
    getSkinModel: function(theme) {
        return new SkinModel(); 
    }
});

/**
* Interface that represents a theme; used in the ThemeModel
* @ibm-api
*/
dojo.declare( "com.ibm.mashups.enabler.theme.Theme", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.Localized], {
  /**
      * @private
      */
    constructor:function () {
    },

   /**
   * Returns the base url of the theme. 
   * @type String 
   * @return base url of the theme. can never be <code>null</code>
   */
    getBaseUrl: function() {
        return new String(); 
    }
});

/**
 * Interface for a skin model. This list model describes a list of skin objects.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.model.SkinModel", com.ibm.mashups.enabler.ListModel, {
    constructor:function () {
    }
});

/**
 * Interface that represents a skin; used in the SkinModel
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.theme.Skin", [com.ibm.mashups.enabler.Identifiable, com.ibm.mashups.enabler.Localized], {
  /**
      * @private
      */
    constructor:function () {
    },

   /**
   * Returns the base url of the skin 
   * @type String 
   * @return base url of the skin. can never be <code>null</code> 
   */
    getBaseUrl: function() {
        return new String(); 
    },

   /**
   * Returns an image which can be used for preview urls 
   * @type String 
   * @return url to an image; may be <code>null</code> 
   */
    getPreviewUrl: function() {
        return new String();
    }
});

/**
 * Interface that acts as base interface for all strategies. It can be set on 
 * models to define the behavior of the model, i.e. how resources are loaded 
 * from the server.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.strategy.Strategy", null, {
  /**
      * @private
      */
    constructor:function () {
    }

});

/**
 * Interface that defines a load ahead strategy for loading elements of a list.
 * When applied to a list model it will load the given number of elements at a 
 * time from the backend.<br>
 * Example; A ListLoadAheadStrategy with interval = 10 will cause the model to 
 * load new data from the backend with every 10th call of the method next() when 
 * using the iterator of the list.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy", com.ibm.mashups.enabler.strategy.Strategy,    {
   /**
    * @param {int} interval number of elements to be loaded ahead from the 
    * backend. must not be <code>null</code> or less than one.
    */
   constructor:function (interval) {
        this.interval = interval;
   },

   /**
   * Returns the number of elements to load ahead 
   * @type int 
   * @return number of elements to load ahead
   */
    getInterval: function() {
        return this.interval;
    }
});

/**
 * Interface that defines a load ahead strategy for loading elements wihin a 
 * tree hierarchy. When applied to a tree model it will load the given number 
 * of elements at a time from the backend.<br>
 * Example; A TreeLoadAheadStrategy with parentLevel = 2 and childrenLevel = 2 will<br>
 * <ul><li>load the parent as well as all the parents of the parent when calling 
 * getParent on a given node and</li>
 * <li>load the children as well as all the children of the child when calling 
 * getChildren on a given node.</li></ul>
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.strategy.TreeLoadAheadStrategy", com.ibm.mashups.enabler.strategy.Strategy,     {
    /**
     * @param {int} parentLevel number of parent levels to be loaded ahead from 
     * the backend. must not be <code>null</code> or less than one.
     * @param {int} childrenLevel number of children levels to be loaded ahead 
     * from the backend. must not be <code>null</code> or less than one.
     */
   constructor:function (parentLevel, childrenLevel) {
        this.parentLevel = parentLevel;
        this.childrenLevel = childrenLevel;
    },

   /**
   * Returns the number of children levels in the hierarchy to load ahead 
   * @type int 
   * @return number of children levels to load ahead
   */
    getChildrenLevel: function() {
        return this.childrenLevel; 
    },

   /**
   * Returns the number of parent levels in the hierarchy to load ahead 
   * @type int 
   * @return number of parent levels to load ahead
   */
    getParentLevel: function() {
        return this.parentLevel; 
    }
});

/**
 * Interface for a community model.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.model.CommunityModel", [com.ibm.mashups.enabler.ListModel],     {
  /**
      * @private
      */
    constructor:function () {
    },

   /**
   * creates a new Community. The created node can be inserted into the model 
   * using an appropriate <code>insert</code> method. The node will not appear 
   * in the model unless it is inserted.
   * @param {object{name: value, ...}} context array of predefined information 
   * used for the creation of the node. May be <code>null</code>. Accepted names are:<br>
   * &nbsp;&nbsp;&nbsp;&nbsp;<code>linkedResource</code> - linked resource object 
   * (mandatory)<br>
   * @type Community 
   * @return created community
   */
    create: function (context) {
        return new Community(); 
    },

   /**
   * Commits the modifications applied to this model and all dependant models.<br> 
   * After the model is committed successfully, this method must not be called 
   * again. The same is true for any other method modifying the model or its nodes.
   * @type Deferred 
   * @return a deferred object used to start this operation. The return value 
   * when executed through the deferred object is <code>null</null>
   */
    commit: function () {
        return new Deferred(); 
    }
});

/**
 * Interface representing a community with participants.
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.community.Community", null,     {
  /**
      * @private
      */
    constructor:function () {
    },

   /**
   * Returns an array of participants. This method will return all participants 
   * of the community when called by the owner of the community, otherwise it  
   * will include only participants that match the requester and their group membership.
   * @type Participant[] 
   * @return array of Participants objects ; never <code>null</code>.
   */
    getParticipants: function() {
        return []; 
    },

   /**
   * Adds an Entity with the given access level 
   * @param {Entity} participant the participant to be added to the community. 
   * Must not be <code>null</code>.
   * @param {AccessLevel} accessLevel the access level for the given participant.
   * Must not be <code>null</code>.
   * @type void
   */
    addParticipant:function (participant, accessLevel) {
    },

   /**
   * Adds an array of Entity objects with the given access level 
   * @param {Entity[]} participants the participants to be added to the community.
   * Must not be <code>null</code>.
   * @param {AccessLevel} accessLevel the access level for the given participant.
   * Must not be <code>null</code>.
   * @type void
   */
    addParticipants:function (participants, accessLevel) {
    },

   /**
   * Removes a participant from this community 
   * @param {Participant} participant the participant to be removed from this 
   * community. Must not be <code>null</code>.
   * @type void
   */
    removeParticipant:function (participant) {
    },

   /**
   * Removes a list of participants from this community 
   * @param {Participant[]} participants the participants to be removed from 
   * this community. Must not be <code>null</code>.
   * type void
   */
    removeParticipants:function (participants) {
    },

   /**
   * Returns a boolean to indicate if the user exists in this community 
   * @param {Entity} user the user to check for. Must not be <code>null</code>.
   * @type Boolean 
   * @return <tt>true</tt> in case the user exists in this community, otherwise 
   * <tt>false</tt>.
   */
    isMember:function (user) {
        return new Boolean();
    },

   /**
   * Returns the owner of this community 
   * @type User 
   * @return the owner of this community. Never <code>null</code>.
   */
    getOwner:function () {
     
        return new User(); 
    }
});

/**
 * AccessLevel constants
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.AccessLevel", null, {
  /**
      * @private
      */
    constructor:function() {
        this.VIEW = 0;
        this.PERSONALIZE = 1;
        this.EDIT = 2;
    },
   /** @type int */
    VIEW : 0,
   /** @type int */
    PERSONALIZE : 1,
   /** @type int */
    EDIT : 2
});

/**
 * Interface representing a participant
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.community.Participant", null, {
  /**
      * @private
      */
    constructor:function() {
    },

   /**
   * Sets the user or group for this participant 
   * @param {Entity} entity the user or group to set. Must not be <code>null</code>.
   * @type void
   */
    setUser: function(entity) {
    },

   /**
   * returns the user or group Entity 
   * @type Entity 
   * @return the enitity of this participant. Never <code>null</code>. 
   */
    getUser: function() {
        return new Entity(); 
    },

   /**
   * sets the access level for this participant 
   * @param {AccessLevel} accessLevel the access level to set. Must not be <code>null</code>.
   * @type void
   */
    setAccessLevel: function(accessLevel) {
    },

   /**
   * returns the access level 
   * @type AccessLevel 
   * @return the access level  of this participant. Never <code>null</code>.
   */
    getAccessLevel: function() {
        return new AccessLevel(); 
    }
});

/**
 * Interface for a user model used to search for user and groups
 */
dojo.declare( "com.ibm.mashups.enabler.model.UserModel", [com.ibm.mashups.enabler.ListModel, com.ibm.mashups.enabler.Locator], {
  /**
      * @private
      */
    constructor:function () {
    },

   /**
   * Returns the current user. 
   * @type Deferred(User) 
   * @return a deferred object used to start this operation. The return value 
   * when executed through the deferred object is the current user or <code>null</code>.
   */
    findCurrentUser: function () {
        return new Deferred();
    },

   /**
   * Returns a user array that matches the LDAP query attributeName=attributeValue 
   * @param {String} attributeName name of the LDAP query
   * @param {String} attributeValue value of the LDAP query
   * @type Deferred(User[]) 
   * @return a deferred object used to start this operation. The return value 
   * when executed through the deferred object is an array of User objects. 
   * Never <code>null</code>.
   */
    findUsersByAttribute: function (attributeName, attributeValue) {
        return new Deferred();
    },

   /**
   * Returns the group with the given id or null if no such group exists 
   * @param {String} id
   * @type Deferred(Group) 
   * @return a deferred object used to start this operation. The return value 
   * when executed through the deferred object is the group with the given id 
   * or <code>null</code>.
   */
    findGroupByID: function (id) {
        return new Deferred(); 
    },

   /**
   * Returns a group array that matches the LDAP query attributeName=attributeValue 
   * @param {String} attributeName name of the LDAP query
   * @param {String} attributeValue value of the LDAP query
   * @type Deferred(Group[]) 
   * @return a deferred object used to start this operation. The return value 
   * when executed through the deferred object is an array of Group objects. 
   * Never <code>null</code>.
   */
    findGroupsByAttribute: function (attributeName,attributeValue) {
        return new Deferred(); 
    },

   /**
   * Returns an array of attributes for the searchable Group attributes in the 
   * user repository. 
   * @type Attribute[] 
   * @return an array of searchable group attributes. Never <code>null</code>.
   */
    findGroupAttributes: function() {
     
        return []; 
    },

   /**
   * Returns an array of attributes for the searchable User attributes in the 
   * user repository. 
   * @type  Attribute[] 
   * @return an array of searchable user attributes. Never <code>null</code>.
   */
    findUserAttributes: function() {
        return []; 
    }
});

/**
 * Interface representing an entity which acts as a base class for a user or a group
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.user.Entity", [com.ibm.mashups.enabler.Identifiable], {
  /**
      * @private
      */
    constructor:function () {
    },

   /**
   * Returns a useful display name. Currently CN + SN for users and CN for groups 
   * @type String 
   * @return the display name; may be <code>null</code>.
   */
    getDisplayName:function () {
        return new String();
    },

   /**
   * Returns the value of user LDAP CN attribute 
   * @type String 
   * @return the value of the CN attritibute; may be <code>null</code>.
   */
    getCN:function () {
        return new String(); 
    },

   /**
   * Returns the value of the given  attribute name 
   * @param {String} name the attribute name to look up
   * @type String 
   * @return the value of the given attribute name; may be <code>null</code>.
   */
    getAttribute:function(name){
        return new String(); 
    },

   /**
   * Returns the attribute names as string array. 
   * @type String[] 
   * @return all attributes names as string array; never  <code>null</code>.
   */
    getAttributeNames:function(){
        return []; 
    }
});

/**
 * Interface representing a user
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.user.User", [com.ibm.mashups.enabler.user.Entity], {
  /**
      * @private
      */
    constructor:function () {
    },

   /**
   * Returns the user LDAP principalName attribute
   * @type String 
   * @return the login name; may be <code>null</code>.
   */
    getLoginName:function () {
        return new String(); 
    },

   /**
   * Returns the user LDAP SN attribute
   * @type String 
   * @return the value of the SN attribute; may be <code>null</code>.
   */
    getSN:function () {
     
        return new String();
    },

   /**
   * Returns a String representation of the user's email 
   * @type String 
   * @return the user's email; may be <code>null</code>.
   */
    getEmail:function () {
        return new String();
    }
});

/**
 * Interface representing a group
 * 
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.user.Group", [com.ibm.mashups.enabler.user.Entity], {
  /**
      * @private
      */
    constructor:function () {
    }
});

/**
 * Interface representing an attribute used in an enitity
 * @ibm-api
 */
dojo.declare( "com.ibm.mashups.enabler.user.Attribute", null, {
  /**
      * @private
      */
   constructor:function () {
   },
   
   /**
    * Returns the attribute name
    * @type     String
    * @return   the attribute name; never <code>null</code>.
    */
   
   getName:function () {
        return new String(); 
   },

   /**
    * Returns the attribute type
    * @type     String
    * @return   the attribute type; never <code>null</code>.
    */
   getType:function () {
        return new String(); 
   }
});

/**
 * Interface representing a model factory
 *
 */
dojo.declare( "com.ibm.mashups.enabler.model.Factory", null, {
    
    /**
     * Returns a catalog category model
     * @ibm-api
     * @type    com.ibm.mashups.enabler.model.CatalogCategoryModel
     * @return  catalog category model, never <code>null</code>
     */
    getCatalogCategoryModel: function() {
        return new com.ibm.mashups.enabler.model.CatalogCategoryModel();
    },

    /**
     * Returns a navigation model
     * 
     * @ibm-api
     * @type    com.ibm.mashups.enabler.model.NavigationModel
     * @return  navigation model, never <code>null</code>
     */
    getNavigationModel: function() {
        return new com.ibm.mashups.enabler.model.NavigationModel(); 
    },

    /**
     * Returns a theme model
     * 
     * @ibm-api
     * @type    com.ibm.mashups.enabler.model.ThemeModel
     * @return  theme model, never <code>null</code> 
     */
    getThemeModel: function() {
        return new com.ibm.mashups.enabler.model.ThemeModel();
    },

//        getPreferenceModel: function(user) {
//            // summary: returns a preference model
//            // name: the name of the user
//            return new com.ibm.mashups.enabler.model.PreferenceModel(user);  /*PreferenceModel @return preference model*/
//        },

    /**
     * Returns a community model
     * 
     * @ibm-api
     * @type    com.ibm.mashups.enabler.model.CommunityModel
     * @return  community model, never <code>null</code>
     */
    getCommunityModel: function() {
        return new com.ibm.mashups.enabler.model.CommunityModel();
    },

    /**
     * Returns a user model
     * @type com.ibm.mashups.enabler.model.UserModel
     * @return user model, never <code>null</code>
     */
    getUserModel: function() {
        return new com.ibm.mashups.enabler.model.UserModel();  
    }
});
